@@ -1,20 +1,5 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "AsyncUserInput.hxx"
@@ -24,15 +9,15 @@
 #include "ncmpc.hxx"
 #include "Point.hxx"
 
-static bool
-ignore_key(int key)
+static constexpr bool
+ignore_key(int key) noexcept
 {
 	return key == ERR || key == '\0';
 }
 
 [[gnu::pure]]
 static Command
-translate_key(int key)
+translate_key(int key) noexcept
 {
 	return GetGlobalKeyBindings().FindKey(key);
 }
@@ -69,7 +54,7 @@ AsyncUserInput::OnSocketReady(unsigned) noexcept
 
 	begin_input_event();
 
-	if (!do_input_event(socket_event.GetEventLoop(), cmd))
+	if (!do_input_event(stdin_event.GetEventLoop(), cmd))
 		return;
 
 	end_input_event();
@@ -77,15 +62,15 @@ AsyncUserInput::OnSocketReady(unsigned) noexcept
 }
 
 AsyncUserInput::AsyncUserInput(EventLoop &event_loop, WINDOW &_w) noexcept
-	:socket_event(event_loop, BIND_THIS_METHOD(OnSocketReady),
+	:stdin_event(event_loop, BIND_THIS_METHOD(OnSocketReady),
 		      FileDescriptor{STDIN_FILENO}),
 	 w(_w)
 {
-	socket_event.ScheduleRead();
+	stdin_event.ScheduleRead();
 }
 
 void
-keyboard_unread(EventLoop &event_loop, int key)
+keyboard_unread(EventLoop &event_loop, int key) noexcept
 {
 	if (ignore_key(key))
 		return;
