@@ -1,20 +1,5 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "ConfigFile.hxx"
 #include "ConfigParser.hxx"
@@ -49,31 +34,22 @@ IsFile(const char *path) noexcept
 std::string
 MakeKeysPath()
 {
-	return MakeUserConfigPath(KEYS_FILENAME);
-}
-
-#ifndef _WIN32
-
-std::string
-GetHomeConfigPath() noexcept
-{
-	const char *home = GetHomeDirectory();
-	if (home == nullptr)
+	const auto dir = GetUserConfigDirectory(PACKAGE);
+	if (dir.empty())
 		return {};
 
-	return BuildPath(home, "." PACKAGE, CONFIG_FILENAME);
+	mkdir(dir.c_str(), 0777);
+	return BuildPath(dir, KEYS_FILENAME);
 }
 
-#endif
-
 std::string
 GetUserConfigPath() noexcept
 {
-	const auto dir = GetHomeConfigDirectory();
+	const auto dir = GetUserConfigDirectory(PACKAGE);
 	if (dir.empty())
 		return {};
 
-	return BuildPath(dir, PACKAGE, CONFIG_FILENAME);
+	return BuildPath(dir, CONFIG_FILENAME);
 }
 
 std::string
@@ -94,30 +70,15 @@ GetSystemConfigPath() noexcept
 #endif
 }
 
-#ifndef _WIN32
-
-[[gnu::pure]]
-static std::string
-GetHomeKeysPath() noexcept
-{
-	const char *home = GetHomeDirectory();
-	if (home == nullptr)
-		return {};
-
-	return BuildPath(home, "." PACKAGE, KEYS_FILENAME);
-}
-
-#endif
-
 [[gnu::pure]]
 static std::string
 GetUserKeysPath() noexcept
 {
-	const auto dir = GetHomeConfigDirectory();
+	const auto dir = GetUserConfigDirectory(PACKAGE);
 	if (dir.empty())
 		return {};
 
-	return BuildPath(dir, PACKAGE, KEYS_FILENAME);
+	return BuildPath(dir, KEYS_FILENAME);
 }
 
 [[gnu::pure]]
@@ -151,13 +112,6 @@ find_config_file() noexcept
 	if (!filename.empty() && IsFile(filename.c_str()))
 		return filename;
 
-#ifndef _WIN32
-	/* check for user configuration ~/.ncmpc/config */
-	filename = GetHomeConfigPath();
-	if (!filename.empty() && IsFile(filename.c_str()))
-		return filename;
-#endif
-
 	/* check for  global configuration SYSCONFDIR/ncmpc/config */
 	filename = GetSystemConfigPath();
 	if (IsFile(filename.c_str()))
@@ -178,13 +132,6 @@ find_keys_file() noexcept
 	if (!filename.empty() && IsFile(filename.c_str()))
 		return filename;
 
-#ifndef _WIN32
-	/* check for  user key bindings ~/.ncmpc/keys */
-	filename = GetHomeKeysPath();
-	if (!filename.empty() && IsFile(filename.c_str()))
-		return filename;
-#endif
-
 	/* check for  global key bindings SYSCONFDIR/ncmpc/keys */
 	filename = GetSystemKeysPath();
 	if (IsFile(filename.c_str()))
