@@ -1,20 +1,5 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "FileBrowserPage.hxx"
 #include "PageMeta.hxx"
@@ -223,9 +208,10 @@ FileBrowserPage::HandleSave(struct mpdclient &c) noexcept
 	}
 
 	if(defaultname)
-		playlist_save(&c, nullptr, Utf8ToLocale(defaultname).c_str());
+		playlist_save(screen, &c, nullptr,
+			      Utf8ToLocale(defaultname).c_str());
 	else
-		playlist_save(&c, nullptr, nullptr);
+		playlist_save(screen, &c, nullptr, nullptr);
 }
 
 void
@@ -258,7 +244,7 @@ FileBrowserPage::HandleDelete(struct mpdclient &c) noexcept
 		snprintf(prompt, sizeof(prompt),
 			 _("Delete playlist %s?"),
 			 Utf8ToLocale(GetUriFilename(mpd_playlist_get_path(playlist))).c_str());
-		bool confirmed = screen_get_yesno(prompt, false);
+		bool confirmed = screen_get_yesno(screen, prompt, false);
 		if (!confirmed) {
 			/* translators: a dialog was aborted by the user */
 			screen_status_message(_("Aborted"));
