@@ -1,29 +1,21 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "ListCursor.hxx"
 #include "Options.hxx"
 
+ListCursor::ListCursor(unsigned _height) noexcept
+	:height(_height),
+	 scroll_offset(ClampScrollOffset(options.scroll_offset, height))
+{
+}
+
 void
 ListCursor::Reset() noexcept
 {
 	selected = 0;
 	range_selection = false;
+	highlight_cursor = false;
 	range_base = 0;
 	start = 0;
 }
@@ -52,6 +44,8 @@ void
 ListCursor::SetHeight(unsigned _height) noexcept
 {
 	height = _height;
+	scroll_offset = ClampScrollOffset(options.scroll_offset, height);
+
 	CheckOrigin();
 }
 
@@ -70,8 +64,10 @@ ListCursor::SetLength(unsigned _length) noexcept
 void
 ListCursor::Center(unsigned n) noexcept
 {
-	if (n > GetHeight() / 2)
-		start = n - GetHeight() / 2;
+	highlight_cursor = false;
+
+	if (n > (GetHeight() - 1) / 2)
+		start = n - (GetHeight() - 1) / 2;
 	else
 		start = 0;
 
@@ -86,18 +82,15 @@ ListCursor::Center(unsigned n) noexcept
 void
 ListCursor::ScrollTo(unsigned n) noexcept
 {
+	highlight_cursor = false;
+
 	int new_start = start;
 
-	if (options.scroll_offset * 2 >= GetHeight())
-		// Center if the offset is more than half the screen
-		new_start = n - GetHeight() / 2;
-	else {
-		if (n < start + options.scroll_offset)
-			new_start = n - options.scroll_offset;
+	if (n < start + scroll_offset)
+		new_start = n - scroll_offset;
 
-		if (n >= start + GetHeight() - options.scroll_offset)
-			new_start = n - GetHeight() + 1 + options.scroll_offset;
-	}
+	if (n >= new_start + GetHeight() - scroll_offset)
+		new_start = n - GetHeight() + 1 + scroll_offset;
 
 	if (new_start + GetHeight() > length)
 		new_start = length - GetHeight();
@@ -122,6 +115,7 @@ void
 ListCursor::MoveCursor(unsigned n) noexcept
 {
 	selected = n;
+	highlight_cursor = false;
 
 	CheckSelected();
 	CheckOrigin();
@@ -130,18 +124,17 @@ ListCursor::MoveCursor(unsigned n) noexcept
 void
 ListCursor::FetchCursor() noexcept
 {
-	/* clamp the scroll-offset setting to slightly less than half
-	   of the screen height */
-	const unsigned scroll_offset = options.scroll_offset * 2 < GetHeight()
-		? options.scroll_offset
-		: std::max(GetHeight() / 2, 1U) - 1;
+	unsigned int target = selected;
 
 	if (start > 0 &&
-	    selected < start + scroll_offset)
-		MoveCursor(start + scroll_offset);
-	else if (start + GetHeight() < length &&
-		 selected > start + GetHeight() - 1 - scroll_offset)
-		MoveCursor(start + GetHeight() - 1 - scroll_offset);
+	    target < start + scroll_offset)
+		target = start + scroll_offset;
+
+	if (start + GetHeight() < length &&
+		 target > start + GetHeight() - 1 - scroll_offset)
+		target = start + GetHeight() - 1 - scroll_offset;
+
+	MoveCursor(target);
 }
 
 ListWindowRange
@@ -186,18 +179,17 @@ ListCursor::MoveCursorTop() noexcept
 {
 	if (start == 0)
 		MoveCursor(start);
+	else if (start + scroll_offset < start + GetHeight() - 1 - scroll_offset)
+		MoveCursor(start + scroll_offset);
 	else
-		if (options.scroll_offset * 2 >= GetHeight())
-			MoveCursor(start + GetHeight() / 2);
-		else
-			MoveCursor(start + options.scroll_offset);
+		MoveCursor(start + GetHeight() - 1 - scroll_offset);
 }
 
 void
 ListCursor::MoveCursorMiddle() noexcept
 {
 	if (length >= GetHeight())
-		MoveCursor(start + GetHeight() / 2);
+		MoveCursor(start + (GetHeight() - 1) / 2);
 	else
 		MoveCursor(length / 2);
 }
@@ -206,13 +198,10 @@ void
 ListCursor::MoveCursorBottom() noexcept
 {
 	if (length >= GetHeight())
-		if (options.scroll_offset * 2 >= GetHeight())
-			MoveCursor(start + GetHeight() / 2);
+		if (start + GetHeight() == length)
+			MoveCursor(length - 1);
 		else
-			if (start + GetHeight() == length)
-				MoveCursor(length - 1);
-			else
-				MoveCursor(start + GetHeight() - 1 - options.scroll_offset);
+			MoveCursor(start + GetHeight() - 1 - scroll_offset);
 	else
 		MoveCursor(length - 1);
 }
@@ -235,6 +224,8 @@ ListCursor::MoveCursorLast() noexcept
 void
 ListCursor::MoveCursorNextPage() noexcept
 {
+	highlight_cursor = false;
+
 	if (GetHeight() < 2)
 		return;
 	if (selected + GetHeight() < length)
@@ -246,6 +237,8 @@ ListCursor::MoveCursorNextPage() noexcept
 void
 ListCursor::MoveCursorPreviousPage() noexcept
 {
+	highlight_cursor = false;
+
 	if (GetHeight() < 2)
 		return;
 	if (selected > GetHeight() - 1)
@@ -257,6 +250,8 @@ ListCursor::MoveCursorPreviousPage() noexcept
 void
 ListCursor::ScrollUp(unsigned n) noexcept
 {
+	highlight_cursor = false;
+
 	if (start > 0) {
 		if (n > start)
 			start = 0;
@@ -270,6 +265,8 @@ ListCursor::ScrollUp(unsigned n) noexcept
 void
 ListCursor::ScrollDown(unsigned n) noexcept
 {
+	highlight_cursor = false;
+
 	if (start + GetHeight() < length) {
 		if (start + GetHeight() + n > length - 1)
 			start = length - GetHeight();
@@ -283,6 +280,8 @@ ListCursor::ScrollDown(unsigned n) noexcept
 void
 ListCursor::ScrollNextPage() noexcept
 {
+	highlight_cursor = false;
+
 	start += GetHeight();
 	if (start + GetHeight() > length)
 		start = length > GetHeight()
@@ -293,6 +292,8 @@ ListCursor::ScrollNextPage() noexcept
 void
 ListCursor::ScrollPreviousPage() noexcept
 {
+	highlight_cursor = false;
+
 	start = start > GetHeight()
 		? start - GetHeight()
 		: 0;
@@ -301,6 +302,8 @@ ListCursor::ScrollPreviousPage() noexcept
 void
 ListCursor::ScrollNextHalfPage() noexcept
 {
+	highlight_cursor = false;
+
 	start += (GetHeight() - 1) / 2;
 	if (start + GetHeight() > length) {
 		start = length > GetHeight()
@@ -312,6 +315,8 @@ ListCursor::ScrollNextHalfPage() noexcept
 void
 ListCursor::ScrollPreviousHalfPage() noexcept
 {
+	highlight_cursor = false;
+
 	start = start > (GetHeight() - 1) / 2
 		? start - (GetHeight() - 1) / 2
 		: 0;
