@@ -1,20 +1,5 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "Instance.hxx"
@@ -53,18 +38,17 @@
 #include <locale.h>
 #endif
 
-#define BUFSIZE 1024
-
 static Instance *global_instance;
-static struct mpdclient *mpd = nullptr;
 
 ScreenManager *screen;
 
 #ifndef NCMPC_MINI
 static void
-update_xterm_title() noexcept
+update_xterm_title(struct mpdclient &client) noexcept
 {
-	const struct mpd_song *song = mpd->GetPlayingSong();
+	const struct mpd_song *song = client.GetPlayingSong();
+
+	static constexpr std::size_t BUFSIZE = 1024;
 
 	char tmp[BUFSIZE];
 	const char *new_title = nullptr;
@@ -86,18 +70,18 @@ update_xterm_title() noexcept
 #endif
 
 static bool
-should_enable_update_timer() noexcept
+should_enable_update_timer(struct mpdclient &client) noexcept
 {
-	return mpd->playing;
+	return client.playing;
 }
 
 static void
-auto_update_timer() noexcept
+auto_update_timer(Instance &instance) noexcept
 {
-	if (should_enable_update_timer())
-		global_instance->EnableUpdateTimer();
+	if (should_enable_update_timer(instance.GetClient()))
+		instance.EnableUpdateTimer();
 	else
-		global_instance->DisableUpdateTimer();
+		instance.DisableUpdateTimer();
 }
 
 void
@@ -109,7 +93,7 @@ Instance::UpdateClient() noexcept
 
 #ifndef NCMPC_MINI
 	if (options.enable_xterm_title)
-		update_xterm_title();
+		update_xterm_title(client);
 #endif
 
 	screen_manager.Update(client, seek);
@@ -133,14 +117,15 @@ mpdclient_connected_callback() noexcept
 {
 #ifndef NCMPC_MINI
 	/* quit if mpd is pre 0.14 - song id not supported by mpd */
-	auto *connection = mpd->GetConnection();
+	auto &client = global_instance->GetClient();
+	auto *connection = client.GetConnection();
 	if (mpd_connection_cmp_server_version(connection, 0, 21, 0) < 0) {
 		const unsigned *version =
 			mpd_connection_get_server_version(connection);
 		screen_status_printf(_("Error: MPD version %d.%d.%d is too old (%s needed)"),
 				     version[0], version[1], version[2],
 				     "0.21.0");
-		mpd->Disconnect();
+		client.Disconnect();
 		doupdate();
 
 		/* try again after 30 seconds */
@@ -154,7 +139,7 @@ mpdclient_connected_callback() noexcept
 
 	global_instance->UpdateClient();
 
-	auto_update_timer();
+	auto_update_timer(*global_instance);
 }
 
 void
@@ -167,7 +152,8 @@ mpdclient_failed_callback() noexcept
 void
 mpdclient_lost_callback() noexcept
 {
-	screen->Update(*mpd, global_instance->GetSeek());
+	screen->Update(global_instance->GetClient(),
+		       global_instance->GetSeek());
 
 	global_instance->ScheduleReconnect(std::chrono::seconds(1));
 }
@@ -179,13 +165,15 @@ mpdclient_lost_callback() noexcept
 void
 mpdclient_idle_callback([[maybe_unused]] unsigned events) noexcept
 {
+	auto &client = global_instance->GetClient();
+
 #ifndef NCMPC_MINI
 	if (options.enable_xterm_title)
-		update_xterm_title();
+		update_xterm_title(client);
 #endif
 
-	screen->Update(*mpd, global_instance->GetSeek());
-	auto_update_timer();
+	screen->Update(client, global_instance->GetSeek());
+	auto_update_timer(*global_instance);
 }
 
 void
@@ -196,7 +184,7 @@ Instance::OnUpdateTimer() noexcept
 
 	UpdateClient();
 
-	if (should_enable_update_timer())
+	if (should_enable_update_timer(client))
 		ScheduleUpdateTimer();
 }
 
@@ -207,10 +195,12 @@ begin_input_event() noexcept
 
 void end_input_event() noexcept
 {
-	screen->Update(*mpd, global_instance->GetSeek());
-	mpd->events = (enum mpd_idle)0;
+	auto &client = global_instance->GetClient();
 
-	auto_update_timer();
+	screen->Update(client, global_instance->GetSeek());
+	client.events = (enum mpd_idle)0;
+
+	auto_update_timer(*global_instance);
 }
 
 bool
@@ -222,7 +212,8 @@ do_input_event(EventLoop &event_loop, Command cmd) noexcept
 	}
 
 	try {
-		screen->OnCommand(*mpd, global_instance->GetSeek(), cmd);
+		screen->OnCommand(global_instance->GetClient(),
+				  global_instance->GetSeek(), cmd);
 	} catch (...) {
 		screen_status_error(std::current_exception());
 		return true;
@@ -241,7 +232,8 @@ void
 do_mouse_event(Point p, mmask_t bstate) noexcept
 {
 	try {
-		screen->OnMouse(*mpd, global_instance->GetSeek(), p, bstate);
+		screen->OnMouse(global_instance->GetClient(),
+				global_instance->GetSeek(), p, bstate);
 	} catch (...) {
 		screen_status_error(std::current_exception());
 	}
@@ -317,7 +309,6 @@ try {
 	/* create the global Instance */
 	Instance instance;
 	global_instance = &instance;
-	mpd = &instance.GetClient();
 	screen = &instance.GetScreenManager();
 
 	AtScopeExit() {
@@ -332,7 +323,7 @@ try {
 	/* attempt to connect */
 	instance.ScheduleReconnect(std::chrono::seconds(0));
 
-	auto_update_timer();
+	auto_update_timer(instance);
 
 #ifndef NCMPC_MINI
 	instance.ScheduleCheckKeyBindings();
