@@ -1,20 +1,5 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Options.hxx"
 #include "Bindings.hxx"
@@ -23,10 +8,10 @@
 #include "charset.hxx"
 #include "ConfigFile.hxx"
 #include "i18n.h"
+#include "util/StringAPI.hxx"
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <string.h>
 
 #define ERROR_UNKNOWN_OPTION    0x01
 #define ERROR_BAD_ARGUMENT      0x02
@@ -62,7 +47,7 @@ static constexpr OptionDefinition option_table[] = {
 #endif
 };
 
-gcc_pure
+[[gnu::pure]]
 static const OptionDefinition *
 FindOption(int s) noexcept
 {
@@ -77,7 +62,7 @@ static const OptionDefinition *
 FindOption(const char *l) noexcept
 {
 	for (const auto &i : option_table)
-		if (strcmp(l, i.longopt) == 0)
+		if (StringIsEqual(l, i.longopt))
 			return &i;
 
 	return nullptr;
@@ -202,17 +187,11 @@ handle_option(int c, const char *arg)
 #ifndef NCMPC_MINI
 		printf("configuration files:\n"
 		       " %s\n"
-#ifndef _WIN32
-		       " %s\n"
-#endif
 		       " %s\n\n",
 		       GetUserConfigPath().c_str(),
-#ifndef _WIN32
-		       GetHomeConfigPath().c_str(),
-#endif
 		       GetSystemConfigPath().c_str());
 
-		if (strcmp("translator-credits", _("translator-credits")) != 0)
+		if (!StringIsEqual("translator-credits", _("translator-credits")))
 			/* To translators: these credits are shown
 			   when ncmpc is started with "--version" */
 			printf("\n%s\n", _("translator-credits"));
@@ -262,7 +241,6 @@ handle_option(int c, const char *arg)
 		GetGlobalKeyBindings().WriteToFile(stdout,
 						   KEYDEF_WRITE_ALL | KEYDEF_COMMENT_ALL);
 		exit(EXIT_SUCCESS);
-		break;
 #endif
 	default:
 		fprintf(stderr,"Unknown Option %c = %s\n", c, arg);
