@@ -1,20 +1,5 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "QueuePage.hxx"
 #include "PageMeta.hxx"
@@ -45,6 +30,10 @@
 
 #ifndef NCMPC_MINI
 #include "hscroll.hxx"
+#include "TableGlue.hxx"
+#include "TableStructure.hxx"
+#include "TableLayout.hxx"
+#include "TablePaint.hxx"
 #endif
 
 #include <mpd/client.h>
@@ -61,6 +50,8 @@ class QueuePage final : public ListPage, ListRenderer, ListText {
 
 #ifndef NCMPC_MINI
 	mutable class hscroll hscroll;
+
+	TableLayout table_layout;
 #endif
 
 	CoarseTimerEvent hide_cursor_timer;
@@ -82,10 +73,14 @@ public:
 #ifndef NCMPC_MINI
 		 hscroll(screen.GetEventLoop(),
 			 w, options.scroll_sep.c_str()),
+		 table_layout(song_table_structure),
 #endif
 		 hide_cursor_timer(screen.GetEventLoop(),
 				   BIND_THIS_METHOD(OnHideCursorTimer))
 	{
+#ifndef NCMPC_MINI
+		table_layout.Calculate(size.width);
+#endif
 	}
 
 private:
@@ -126,6 +121,14 @@ public:
 	/* virtual methods from class Page */
 	void OnOpen(struct mpdclient &c) noexcept override;
 	void OnClose() noexcept override;
+
+	void OnResize(Size size) noexcept override {
+		ListPage::OnResize(size);
+#ifndef NCMPC_MINI
+		table_layout.Calculate(size.width);
+#endif
+	}
+
 	void Paint() const noexcept override;
 	bool PaintStatusBarOverride(const Window &window) const noexcept override;
 	void Update(struct mpdclient &c, unsigned events) noexcept override;
@@ -249,12 +252,14 @@ add_dir(Completion &completion, const char *dir,
 }
 
 class DatabaseCompletion final : public Completion {
+	ScreenManager &screen;
 	struct mpdclient &c;
 	std::set<std::string> dir_list;
 
 public:
-	explicit DatabaseCompletion(struct mpdclient &_c) noexcept
-		:c(_c) {}
+	DatabaseCompletion(ScreenManager &_screen,
+			   struct mpdclient &_c) noexcept
+		:screen(_screen), c(_c) {}
 
 protected:
 	/* virtual methods from class Completion */
@@ -281,7 +286,7 @@ DatabaseCompletion::Post(const char *line, Range range) noexcept
 {
 	if (range.begin() != range.end() &&
 	    std::next(range.begin()) != range.end())
-		screen_display_completion_list(range);
+		screen_display_completion_list(screen, range);
 
 	if (line && line[0] && line[strlen(line) - 1] == '/') {
 		/* add directory content to list */
@@ -294,18 +299,18 @@ DatabaseCompletion::Post(const char *line, Range range) noexcept
 #endif
 
 static int
-handle_add_to_playlist(struct mpdclient *c)
+handle_add_to_playlist(ScreenManager &screen, struct mpdclient *c)
 {
 #ifndef NCMPC_MINI
 	/* initialize completion support */
-	DatabaseCompletion _completion(*c);
+	DatabaseCompletion _completion{screen, *c};
 	Completion *completion = &_completion;
 #else
 	Completion *completion = nullptr;
 #endif
 
 	/* get path */
-	auto path = screen_readln(_("Add"),
+	auto path = screen_readln(screen, _("Add"),
 				  nullptr,
 				  nullptr,
 				  completion);
@@ -381,6 +386,15 @@ QueuePage::PaintListItem(WINDOW *w, unsigned i, unsigned y, unsigned width,
 	assert(i < playlist->size());
 	const auto &song = (*playlist)[i];
 
+#ifndef NCMPC_MINI
+	if (!song_table_structure.columns.empty()) {
+		PaintTableRow(w, width, selected,
+			      (int)mpd_song_get_id(&song) == current_song_id,
+			      song, table_layout);
+		return;
+	}
+#endif
+
 	class hscroll *row_hscroll = nullptr;
 #ifndef NCMPC_MINI
 	row_hscroll = selected && options.scroll && lw.GetCursorIndex() == i
@@ -615,11 +629,11 @@ QueuePage::OnCommand(struct mpdclient &c, Command cmd)
 	}
 
 	case Command::SAVE_PLAYLIST:
-		playlist_save(&c, nullptr, nullptr);
+		playlist_save(screen, &c, nullptr, nullptr);
 		return true;
 
 	case Command::ADD:
-		handle_add_to_playlist(&c);
+		handle_add_to_playlist(screen, &c);
 		return true;
 
 	case Command::SHUFFLE: {
