@@ -1,20 +1,5 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "SearchPage.hxx"
 #include "PageMeta.hxx"
@@ -29,6 +14,7 @@
 #include "screen_utils.hxx"
 #include "FileListPage.hxx"
 #include "filelist.hxx"
+#include "util/StringAPI.hxx"
 
 #include <iterator>
 
@@ -36,9 +22,7 @@
 
 enum {
 	SEARCH_URI = MPD_TAG_COUNT + 100,
-#if LIBMPDCLIENT_CHECK_VERSION(2,10,0)
 	SEARCH_MODIFIED,
-#endif
 	SEARCH_ARTIST_TITLE,
 };
 
@@ -63,25 +47,23 @@ static constexpr struct {
 static int
 search_get_tag_id(const char *name)
 {
-	if (strcasecmp(name, "file") == 0 ||
-	    strcasecmp(name, _("file")) == 0)
+	if (StringIsEqualIgnoreCase(name, "file") ||
+	    StringIsEqualIgnoreCase(name, _("file")))
 		return SEARCH_URI;
 
-#if LIBMPDCLIENT_CHECK_VERSION(2,10,0)
-	if (strcasecmp(name, "modified") == 0)
+	if (StringIsEqualIgnoreCase(name, "modified"))
 		return SEARCH_MODIFIED;
-#endif
 
 	for (unsigned i = 0; search_tag[i].name != nullptr; ++i)
-		if (strcasecmp(search_tag[i].name, name) == 0 ||
-		    strcasecmp(search_tag[i].localname, name) == 0)
+		if (StringIsEqualIgnoreCase(search_tag[i].name, name) ||
+		    StringIsEqualIgnoreCase(search_tag[i].localname, name))
 			return search_tag[i].tag_type;
 
 	return -1;
 }
 
 struct SearchMode {
-	enum mpd_tag_type table;
+	int table;
 	const char *label;
 };
 
@@ -89,8 +71,8 @@ static constexpr SearchMode mode[] = {
 	{ MPD_TAG_TITLE, N_("Title") },
 	{ MPD_TAG_ARTIST, N_("Artist") },
 	{ MPD_TAG_ALBUM, N_("Album") },
-	{ (enum mpd_tag_type)SEARCH_URI, N_("Filename") },
-	{ (enum mpd_tag_type)SEARCH_ARTIST_TITLE, N_("Artist + Title") },
+	{ SEARCH_URI, N_("Filename") },
+	{ SEARCH_ARTIST_TITLE, N_("Artist + Title") },
 	{ MPD_TAG_COUNT, nullptr }
 };
 
@@ -103,11 +85,7 @@ static const char *const help_text[] = {
 	"",
 	"Advanced  -  <tag>:<search term> [<tag>:<search term>...]",
 	"		Example: artist:radiohead album:pablo honey",
-#if LIBMPDCLIENT_CHECK_VERSION(2,10,0)
 	"		Example: modified:14d (units: s, M, h, d, m, y)",
-#else
-	"		(\"modified:\" requires libmpdclient 2.10)",
-#endif
 	"",
 	"		Available tags: artist, album, title, track,",
 	"		name, genre, date composer, performer, comment, file",
@@ -126,7 +104,7 @@ public:
 			      !options.search_format.empty()
 			      ? options.search_format.c_str()
 			      : options.list_format.c_str()) {
-		lw.DisableCursor();
+		lw.HideCursor();
 		lw.SetLength(std::size(help_text));
 	}
 
@@ -229,8 +207,6 @@ search_simple_query(struct mpd_connection *connection, bool exact_match,
 	}
 }
 
-#if LIBMPDCLIENT_CHECK_VERSION(2,10,0)
-
 /**
  * Throws on error.
  */
@@ -290,8 +266,6 @@ ParseModifiedSince(const char *s)
 	return time(nullptr) - value;
 }
 
-#endif
-
 /*-----------------------------------------------------------------------
  * NOTE: This code exists to test a new search ui,
  *       Its ugly and MUST be redesigned before the next release!
@@ -369,12 +343,10 @@ try {
 			mpd_search_add_uri_constraint(connection,
 						      MPD_OPERATOR_DEFAULT,
 						      value.c_str());
-#if LIBMPDCLIENT_CHECK_VERSION(2,10,0)
 		else if (table[i] == SEARCH_MODIFIED)
 			mpd_search_add_modified_since_constraint(connection,
 								 MPD_OPERATOR_DEFAULT,
 								 ParseModifiedSince(value.c_str()));
-#endif
 		else
 			mpd_search_add_tag_constraint(connection,
 						      MPD_OPERATOR_DEFAULT,
@@ -423,7 +395,7 @@ SearchPage::Reload(struct mpdclient &c)
 	if (pattern.empty())
 		return;
 
-	lw.EnableCursor();
+	lw.ShowCursor();
 	filelist = do_search(&c, pattern.c_str());
 	if (filelist == nullptr)
 		filelist = std::make_unique<FileList>();
@@ -442,7 +414,7 @@ SearchPage::Start(struct mpdclient &c)
 
 	Clear(true);
 
-	pattern = screen_readln(_("Search"),
+	pattern = screen_readln(screen, _("Search"),
 				nullptr,
 				&search_history,
 				nullptr);
