@@ -1,20 +1,5 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "SongPage.hxx"
 #include "PageMeta.hxx"
@@ -31,6 +16,7 @@
 #include "time_format.hxx"
 #include "mpdclient.hxx"
 #include "util/LocaleString.hxx"
+#include "util/StringAPI.hxx"
 #include "util/StringStrip.hxx"
 
 #include <mpd/client.h>
@@ -41,7 +27,6 @@
 #include <string>
 
 #include <assert.h>
-#include <string.h>
 #include <time.h>
 
 enum {
@@ -458,8 +443,8 @@ SongPage::Update(struct mpdclient &c, unsigned) noexcept
 
 	if (selected_song != nullptr &&
 	    (playing_song == nullptr ||
-	     strcmp(mpd_song_get_uri(selected_song),
-		    mpd_song_get_uri(playing_song)) != 0)) {
+	     !StringIsEqual(mpd_song_get_uri(selected_song),
+			    mpd_song_get_uri(playing_song)))) {
 		lines.emplace_back(_("Selected song"));
 		AddSong(selected_song);
 		lines.emplace_back(std::string());
