@@ -1,31 +1,5 @@
-/*
- * Copyright 2012-2020 Max Kellermann <max.kellermann@gmail.com>
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * - Redistributions of source code must retain the above copyright
- * notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above copyright
- * notice, this list of conditions and the following disclaimer in the
- * documentation and/or other materials provided with the
- * distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
- * FOUNDATION OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+// SPDX-License-Identifier: BSD-2-Clause
+// author: Max Kellermann <max.kellermann@gmail.com>
 
 #include "FileDescriptor.hxx"
 #include "UniqueFileDescriptor.hxx"
@@ -41,11 +15,6 @@
 #include <poll.h>
 #endif
 
-#ifdef __linux__
-#include <sys/eventfd.h>
-#include <sys/signalfd.h>
-#endif
-
 #ifndef O_NOCTTY
 #define O_NOCTTY 0
 #endif
@@ -185,7 +154,7 @@ FileDescriptor::CreatePipe(FileDescriptor &r, FileDescriptor &w) noexcept
 #ifdef _WIN32
 
 void
-FileDescriptor::SetBinaryMode() noexcept
+FileDescriptor::SetBinaryMode() const noexcept
 {
 	_setmode(fd, _O_BINARY);
 }
@@ -209,7 +178,7 @@ FileDescriptor::CreatePipeNonBlock(FileDescriptor &r,
 }
 
 void
-FileDescriptor::SetNonBlocking() noexcept
+FileDescriptor::SetNonBlocking() const noexcept
 {
 	assert(IsDefined());
 
@@ -218,7 +187,7 @@ FileDescriptor::SetNonBlocking() noexcept
 }
 
 void
-FileDescriptor::SetBlocking() noexcept
+FileDescriptor::SetBlocking() const noexcept
 {
 	assert(IsDefined());
 
@@ -227,7 +196,7 @@ FileDescriptor::SetBlocking() noexcept
 }
 
 void
-FileDescriptor::EnableCloseOnExec() noexcept
+FileDescriptor::EnableCloseOnExec() const noexcept
 {
 	assert(IsDefined());
 
@@ -236,7 +205,7 @@ FileDescriptor::EnableCloseOnExec() noexcept
 }
 
 void
-FileDescriptor::DisableCloseOnExec() noexcept
+FileDescriptor::DisableCloseOnExec() const noexcept
 {
 	assert(IsDefined());
 
@@ -251,7 +220,7 @@ FileDescriptor::Duplicate() const noexcept
 }
 
 bool
-FileDescriptor::CheckDuplicate(FileDescriptor new_fd) noexcept
+FileDescriptor::CheckDuplicate(FileDescriptor new_fd) const noexcept
 {
 	if (*this == new_fd) {
 		DisableCloseOnExec();
@@ -262,30 +231,8 @@ FileDescriptor::CheckDuplicate(FileDescriptor new_fd) noexcept
 
 #endif
 
-#ifdef __linux__
-
 bool
-FileDescriptor::CreateEventFD(unsigned initval) noexcept
-{
-	fd = ::eventfd(initval, EFD_NONBLOCK|EFD_CLOEXEC);
-	return fd >= 0;
-}
-
-bool
-FileDescriptor::CreateSignalFD(const sigset_t *mask) noexcept
-{
-	int new_fd = ::signalfd(fd, mask, SFD_NONBLOCK|SFD_CLOEXEC);
-	if (new_fd < 0)
-		return false;
-
-	fd = new_fd;
-	return true;
-}
-
-#endif
-
-bool
-FileDescriptor::Rewind() noexcept
+FileDescriptor::Rewind() const noexcept
 {
 	assert(IsDefined());
 
@@ -302,7 +249,7 @@ FileDescriptor::GetSize() const noexcept
 }
 
 void
-FileDescriptor::FullRead(void *_buffer, std::size_t length)
+FileDescriptor::FullRead(void *_buffer, std::size_t length) const
 {
 	auto buffer = (std::byte *)_buffer;
 
@@ -320,7 +267,7 @@ FileDescriptor::FullRead(void *_buffer, std::size_t length)
 }
 
 void
-FileDescriptor::FullWrite(const void *_buffer, std::size_t length)
+FileDescriptor::FullWrite(const void *_buffer, std::size_t length) const
 {
 	auto buffer = (const std::byte *)_buffer;
 
