@@ -1,20 +1,5 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "save_playlist.hxx"
 #include "db_completion.hxx"
@@ -25,7 +10,6 @@
 #include "mpdclient.hxx"
 #include "Completion.hxx"
 #include "screen_utils.hxx"
-#include "util/Compiler.h"
 
 #include <mpd/client.h>
 
@@ -34,11 +18,13 @@
 #ifndef NCMPC_MINI
 
 class PlaylistNameCompletion final : public Completion {
+	ScreenManager &screen;
 	struct mpdclient &c;
 
 public:
-	explicit PlaylistNameCompletion(struct mpdclient &_c) noexcept
-		:c(_c) {}
+	PlaylistNameCompletion(ScreenManager &_screen,
+			       struct mpdclient &_c) noexcept
+		:screen(_screen), c(_c) {}
 
 protected:
 	/* virtual methods from class Completion */
@@ -47,7 +33,7 @@ protected:
 };
 
 void
-PlaylistNameCompletion::Pre(gcc_unused const char *value) noexcept
+PlaylistNameCompletion::Pre([[maybe_unused]] const char *value) noexcept
 {
 	if (empty()) {
 		/* create completion list */
@@ -56,18 +42,19 @@ PlaylistNameCompletion::Pre(gcc_unused const char *value) noexcept
 }
 
 void
-PlaylistNameCompletion::Post(gcc_unused const char *value,
+PlaylistNameCompletion::Post([[maybe_unused]] const char *value,
 			     Range range) noexcept
 {
 	if (range.begin() != range.end() &&
 	    std::next(range.begin()) != range.end())
-		screen_display_completion_list(range);
+		screen_display_completion_list(screen, range);
 }
 
 #endif
 
 int
-playlist_save(struct mpdclient *c, const char *name,
+playlist_save(ScreenManager &screen, struct mpdclient *c,
+	      const char *name,
 	      const char *defaultname) noexcept
 {
 	std::string filename;
@@ -77,12 +64,12 @@ playlist_save(struct mpdclient *c, const char *name,
 		Completion *completion = nullptr;
 #else
 		/* initialize completion support */
-		PlaylistNameCompletion _completion(*c);
+		PlaylistNameCompletion _completion{screen, *c};
 		auto *completion = &_completion;
 #endif
 
 		/* query the user for a filename */
-		filename = screen_readln(_("Save queue as"),
+		filename = screen_readln(screen, _("Save queue as"),
 					 defaultname,
 					 nullptr,
 					 completion);
@@ -105,7 +92,7 @@ playlist_save(struct mpdclient *c, const char *name,
 			char prompt[256];
 			snprintf(prompt, sizeof(prompt),
 				 _("Replace %s?"), filename.c_str());
-			bool replace = screen_get_yesno(prompt, false);
+			bool replace = screen_get_yesno(screen, prompt, false);
 			if (!replace) {
 				screen_status_message(_("Aborted"));
 				return -1;
