@@ -1,20 +1,5 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "screen.hxx"
 #include "PageMeta.hxx"
@@ -30,11 +15,10 @@
 #include "player_command.hxx"
 #include "SongPage.hxx"
 #include "LyricsPage.hxx"
+#include "util/StringAPI.hxx"
 
 #include <mpd/client.h>
 
-#include <string.h>
-
 ScreenManager::PageMap::iterator
 ScreenManager::MakePage(const PageMeta &sf) noexcept
 {
@@ -95,14 +79,14 @@ ScreenManager::Swap(struct mpdclient &c, const struct mpd_song *song) noexcept
 		Switch(*mode_fn_prev, c);
 }
 
-gcc_pure
+[[gnu::pure]]
 static int
 find_configured_screen(const char *name) noexcept
 {
 	unsigned i;
 
 	for (i = 0; i < options.screen_list.size(); ++i)
-		if (strcmp(options.screen_list[i].c_str(), name) == 0)
+		if (StringIsEqual(options.screen_list[i].c_str(), name))
 			return i;
 
 	return -1;
