@@ -1,20 +1,5 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "screen_find.hxx"
 #include "screen_utils.hxx"
@@ -58,7 +43,7 @@ screen_find(ScreenManager &screen, ListWindow &lw, Command findcmd,
 		if (screen.findbuf.empty()) {
 			char *value = options.find_show_last_pattern
 				? (char *) -1 : nullptr;
-			screen.findbuf=screen_readln(prompt,
+			screen.findbuf=screen_readln(screen, prompt,
 						     value,
 						     &screen.find_history,
 						     nullptr);
@@ -106,7 +91,7 @@ screen_jump(ScreenManager &screen, ListWindow &lw,
 	char *iter = search_str;
 
 	while(1) {
-		key = screen_getch(buffer);
+		key = screen_getch(screen, buffer);
 		/* if backspace or delete was pressed, process instead of ending loop */
 		if (key == KEY_BACKSPACE || key == KEY_DC) {
 			const char *prev = PrevCharMB(buffer, iter);
